/*Copyright (C) 2017 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
#include <ctime>

#define BUFFERSIZE 300
#define ETALONTEMP "/tmp/abashin.ru.etalon.temp"

using namespace std;

struct DOUBLE_ETALON {
  time_t current_time;
  int intensity;
  int key1;
  int retaining1;
  int space1;
  int key2;
  int retaining2;
};

struct MOUSEPRESSETALON {
  time_t current_time;
  int key;
  int intensity;
  int retaining1;
};

struct KEYBOARD_DOUBLE_ETALON {
  int key1;
  int retaining1_min;
  int retaining1_max;
  int space1_min;
  int space1_max;
  int key2;
  int retaining2_min;
  int retaining2_max;
};

struct MOUSE_PRESS_ETALON {
  int key;
  int retaining1_min;
  int retaining1_max;
};

void create_etalon(char * etalondata, char * etalon);
int calculate_and_save_etalon (vector<MOUSEPRESSETALON> * mouse_etalon,
                   vector<DOUBLE_ETALON> * keyboard_etalon, char * etalon);


int main (int argc, char * argv[]) {
  if (argc != 4) return 0;
  char * SAVEDATA = argv[1];
  char * ETALONDATA = argv[2];
  char * ETALON = argv[3];

  ifstream ifs;
  string buff;
  buff.clear();
  buff = buff + SAVEDATA;
  ifs.open(buff, ifstream::in);
  if(!ifs) return 0;
  char buffer [BUFFERSIZE] = {0};
  ofstream ofs;
  buff.clear();
  buff = buff + ETALONDATA;
  ofs.open(buff, ofstream::out | ofstream::app);
  if (!ofs) return 0;
  ifs.getline ((char *)&buffer,BUFFERSIZE-1);
  while (ifs.good()) {
      ofs <<  (char *)&buffer << endl;
      ifs.getline ((char *)&buffer,BUFFERSIZE-1);
  }
  ifs.close();
  ofs.close();

  buff.clear();
  buff = buff + SAVEDATA;
  ofs.open(buff, ofstream::out);  
  if (!ofs) return 0;
  ofs.close();

  buff.clear();
  buff = buff + ETALON;
  ifs.open(buff, ifstream::in);
  if (ifs) {
    ifs.getline ((char *)&buffer,BUFFERSIZE-1);
    ifs.close();
    if (time(NULL) - atol((char *)&buffer) < 3*2592000)
        return 0;
  }

  buff.clear();
  buff = buff + ETALONDATA;
  ifs.open(buff, ifstream::in);
  int count = 0;
  if (!ifs) return 0;
  do {
    ifs.getline ((char *)&buffer,BUFFERSIZE-1);
    count++;
  } while (ifs.good());
  ifs.close();
  if (count < 10000) return 0;
  create_etalon (ETALONDATA, ETALON);
  return 0;
}


void create_etalon(char * etalondata, char * etalon) {
  vector<MOUSEPRESSETALON> mouse_etalon;
  mouse_etalon.reserve(5000000);
  vector<DOUBLE_ETALON> keyboard_etalon;
  keyboard_etalon.reserve(5000000);

  string buff;
  int code = 0;
  int previous_code = 0;
  int intensity = 0;
  time_t current_time = 0;
  time_t start_time = 0;

  ifstream data_file;
  buff.clear();
  buff = buff + etalondata;
  data_file.open(buff, ifstream::in);

  data_file >> code >> start_time;
  current_time = start_time;
  previous_code = code;
  while (data_file.good()) {
    data_file >> code;
    if (code == 0) data_file >> current_time;
    if (code == 1)
      if (code != previous_code) {
        data_file >> intensity;
      }
      else {
        struct DOUBLE_ETALON sde = {0};
        data_file >> sde.intensity;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        data_file >> sde.key1;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        data_file >> sde.retaining1;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        data_file >> sde.space1;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        data_file >> sde.key2;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        data_file >> sde.retaining2;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        sde.current_time = current_time;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        keyboard_etalon.push_back(sde);
      }
    else;
    if (code == 2)
      if (code != previous_code) {
        data_file >> intensity;
      }
      else {
        struct MOUSEPRESSETALON sme = {0};
        data_file >> sme.intensity;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        data_file >> sme.key;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        data_file >> sme.retaining1;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        sme.current_time = current_time;
        if (!data_file) {
          previous_code = code;
          continue;
        }
        mouse_etalon.push_back(sme);
      }
    else;
    previous_code = code;
  }
  data_file.close();
  int function_result = calculate_and_save_etalon (&mouse_etalon,
                        &keyboard_etalon, etalon);
  if (function_result == 0) {
    string command;
    command = command + "rm -f " + etalondata;
    int result = system(command.c_str());
    result++;
  }
}


int calculate_and_save_etalon (vector<MOUSEPRESSETALON> * mouse_etalon,
                   vector<DOUBLE_ETALON> * keyboard_etalon, char * etalon) {
  int how_many_m = 0;
  int how_many_k = 0;
  string buff;
  vector <MOUSE_PRESS_ETALON> me;
  me.reserve(1000);
  vector <KEYBOARD_DOUBLE_ETALON> ke;
  ke.reserve(1000);

  ofstream etalon_file;
  etalon_file.open(ETALONTEMP, ofstream::out);
  if (!etalon_file) {
    cout << " Error can`t create etalon!" << endl;
    return 1;
  }
  etalon_file << time(NULL) << endl;

  struct MOUSE_PRESS_ETALON mpe = {0};
  me.push_back(mpe);
  int m_device_count = 0;
  for (vector<MOUSEPRESSETALON>::iterator count = mouse_etalon->begin();
    count!=mouse_etalon->end(); count++) {
    if (count->current_time != 0) {
      int intensity = 0;
      int value = 0;
      me.at(m_device_count).key = count->key;
      me.at(m_device_count).retaining1_min = count->retaining1;
      me.push_back(mpe);
      for (vector<MOUSEPRESSETALON>::iterator second_count = count;
        second_count!=mouse_etalon->end(); second_count++) {
        if (me.at(m_device_count).key == second_count->key) {
          if (second_count->intensity != 0 &&
              second_count->retaining1 < 1000) {
            value += second_count->retaining1 *
                     second_count->intensity;
            intensity += second_count->intensity;
            if (me.at(m_device_count).retaining1_min > second_count->retaining1)
                me.at(m_device_count).retaining1_min = second_count->retaining1;
          }
          second_count->current_time = 0;
          second_count->intensity = 0;
          second_count->key = 0;
          second_count->retaining1 = 0;
        }
      }
      if (intensity == 0) intensity++;
      int average = value / intensity;
      me.at(m_device_count).retaining1_min = me.at(m_device_count).retaining1_min +
                            (average - me.at(m_device_count).retaining1_min)/2;
      me.at(m_device_count).retaining1_max = average +
                            (average - me.at(m_device_count).retaining1_min);
      if (me.at(m_device_count).key != 0 && me.at(m_device_count).retaining1_min <
          me.at(m_device_count).retaining1_max) {
        etalon_file << "2 " << me.at(m_device_count).key << " " <<
                  me.at(m_device_count).retaining1_min << " " <<
                  me.at(m_device_count).retaining1_max << endl;
        how_many_m++;
      }
      m_device_count++;
    }
  }

  struct KEYBOARD_DOUBLE_ETALON kde = {0};
  ke.push_back(kde);
  int k_device_count = 0;
  for (vector<DOUBLE_ETALON>::iterator count = keyboard_etalon->begin();
  count!=keyboard_etalon->end(); count++) {
    if (count->current_time != 0) {
      int intensity = 0;
      int value_retaining1 = 0;
      int value_space1 = 0;
      int value_retaining2 = 0;
      ke.at(k_device_count).key1 = count->key1;
      ke.at(k_device_count).key2 = count->key2;
      ke.at(k_device_count).retaining1_min = count->retaining1;
      ke.at(k_device_count).space1_min = count->space1;
      ke.at(k_device_count).retaining2_min = count->retaining2;
      ke.push_back(kde);
      for (vector<DOUBLE_ETALON>::iterator second_count = count;
        second_count!=keyboard_etalon->end(); second_count++) {
        if (ke.at(k_device_count).key1 == second_count->key1 &&
            ke.at(k_device_count).key2 == second_count->key2) {
          if (second_count->intensity != 0 &&
              second_count->retaining1 < 1000 &&
              second_count->space1 < 1000 &&
              second_count->retaining2 < 1000) {
            value_retaining1 += second_count->retaining1 * 
                     second_count->intensity;
            value_space1 += second_count->space1 *
                     second_count->intensity;
            value_retaining2 += second_count->retaining2 *
                     second_count->intensity;
            intensity += second_count->intensity;
            if (ke.at(k_device_count).retaining1_min >
                second_count->retaining1)
                ke.at(k_device_count).retaining1_min = 
                second_count->retaining1;
            if (ke.at(k_device_count).space1_min >
                second_count->space1)
                ke.at(k_device_count).space1_min = 
                second_count->space1;
            if (ke.at(k_device_count).retaining2_min >
                second_count->retaining2)
                ke.at(k_device_count).retaining2_min = 
                second_count->retaining2;
          }
          second_count->current_time = 0;
          second_count->intensity = 0;
          second_count->key1 = 0;
          second_count->retaining1 = 0;
          second_count->space1 = 0;
          second_count->key2 = 0;
          second_count->retaining2 = 0;
        }
      }

      if (intensity == 0) intensity++;
      int average_retaining1 = 0;
      int average_space1 = 0;
      int average_retaining2 = 0;

      average_retaining1 = value_retaining1 / intensity;
      average_space1 = value_space1 / intensity;
      average_retaining2 = value_retaining2 / intensity;

      ke.at(k_device_count).retaining1_min = ke.at(k_device_count).retaining1_min +
          (average_retaining1 - ke.at(k_device_count).retaining1_min)/2;
      ke.at(k_device_count).space1_min = ke.at(k_device_count).space1_min +
          (average_space1 - ke.at(k_device_count).space1_min)/2;
      ke.at(k_device_count).retaining2_min = ke.at(k_device_count).retaining2_min +
          (average_retaining2 - ke.at(k_device_count).retaining2_min)/2;

      ke.at(k_device_count).retaining1_max = average_retaining1 + (average_retaining1 -
                                        ke.at(k_device_count).retaining1_min);
      ke.at(k_device_count).space1_max = average_space1 + (average_space1 - 
                                        ke.at(k_device_count).space1_min);
      ke.at(k_device_count).retaining2_max = average_retaining2 + (average_retaining2 -
                                        ke.at(k_device_count).retaining2_min);

      if (ke.at(k_device_count).key1 != 0 && ke.at(k_device_count).key2 != 0 &&
      ke.at(k_device_count).retaining1_min < ke.at(k_device_count).retaining1_max &&
      ke.at(k_device_count).space1_min < ke.at(k_device_count).space1_max &&
      ke.at(k_device_count).retaining2_min < ke.at(k_device_count).retaining2_max) {
        etalon_file << "1 " << ke.at(k_device_count).key1 << " " <<
                  ke.at(k_device_count).retaining1_min << " " <<
                  ke.at(k_device_count).retaining1_max << " " <<
                  ke.at(k_device_count).space1_min << " " <<
                  ke.at(k_device_count).space1_max << " " <<
                  ke.at(k_device_count).key2 << " " <<
                  ke.at(k_device_count).retaining2_min << " " <<
                  ke.at(k_device_count).retaining2_max << " " << endl;
        how_many_k++;
      }
      k_device_count++;
    }
  }
  etalon_file.flush();
  etalon_file.close();

  if ((how_many_m > 0) || (how_many_k > 0)) {
    string command;
    command = command + "mv -f " + ETALONTEMP + " " + etalon;
    int result = system(command.c_str());
    result++;
  }
  return 0;
}
