/*Copyright (C) 2017 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
#include <fstream>
#include <cstdlib>
#include <vector>
#include <string>
#include <ctime>

using namespace std;

#define REPORTFILE "/.abashin.ru/abashin.ru.html"
#define MESSAGETYPEOUTPUT "Input information / Communication"
#define MESSAGETYPEINPUT "Obtaining information / Information processing"
#define MESSAGETYPEREST "Rest"

struct REPORT_RECORD {
  time_t start_time;
  time_t stop_time;
  int intensity;
  char rest_or_work;
  float pfs;
  int pfs_count;
};

struct HUMAN_STATUS {
  time_t current_time;
  int intensity;
  float pfs;
  int device_id;
};

void human_status_to_report_record (vector <HUMAN_STATUS> * hs,
                                    vector <REPORT_RECORD> *rr);
void get_earth_day (time_t current_time, time_t * begin_time, time_t * end_time);
void print_report (vector <HUMAN_STATUS> * hs, string * file_report,
                   time_t current_time);
int when_a_break (vector <HUMAN_STATUS> * hs, time_t current_time);
void warning (char type);

int main (int argc, char * argv[]) {
  if (argc != 2) return 0;
  char * HUMANSTATUSLOG = argv[1];
  struct HUMAN_STATUS shs = {0};

  time_t current_time = time(NULL);
  time_t begin_time = 0;
  time_t end_time = 0;
  get_earth_day (current_time, &begin_time, &end_time);

  vector <HUMAN_STATUS> hs;
  hs.reserve(100);

  ifstream hs_file;
  string buff;
  buff.clear();
  buff = buff + HUMANSTATUSLOG;
  hs_file.open(buff, ifstream::in);
  if (!hs_file) return 0;
  do {
    hs_file >> shs.current_time >> shs.intensity >> shs.pfs >> shs.device_id;
    if ((shs.current_time >= begin_time) && (shs.current_time <= end_time))
    hs.push_back(shs);
  }
  while (hs_file.good());
  hs_file.close();

  string report_path;
  report_path.clear();
  report_path = report_path + getenv("HOME") + REPORTFILE;
  print_report ((vector <HUMAN_STATUS> *)&hs, (string *) &report_path, current_time);

  int before_break = when_a_break ((vector <HUMAN_STATUS> *) &hs, current_time);

  if (before_break < 5) warning (1);
  else if (before_break <= 0) warning (2);
  if (before_break >= 90) warning(3);

  return 0;
}

void get_earth_day (time_t current_time, time_t * begin_time, time_t * end_time) {
  struct tm * timeinfo;
  time(&current_time);
  timeinfo = localtime(&current_time);
  timeinfo->tm_hour = 0;
  timeinfo->tm_min = 0;
  timeinfo->tm_sec = 0;
  *begin_time = mktime (timeinfo);
  timeinfo->tm_hour = 23;
  timeinfo->tm_min = 59;
  timeinfo->tm_sec = 59;
  *end_time = mktime (timeinfo);
}

void print_report (vector <HUMAN_STATUS> * hs, string * file_report,
                   time_t current_time) {
  fstream report;
  report.open(file_report->c_str(), fstream::out | fstream::trunc);
  report << "<!DOCTYPE html>" << endl;
  report << "<html lang=\"ru\">" << endl;
  report << "<head>" << endl;
  report << "<meta charset=\"utf-8\">" << endl;
  report << "<title>Abashin.ru</title>" << endl;
  report << "<style type=\"text/css\">" << endl;
  report << "h1 { color: #aa0000 }" << endl;
  report << "#table {" << endl;
  report << "display: table;" << endl;
  report << "width:100%;" << endl;
  report << "border:1px solid white;" << endl;
  report << "border-collapse:collapse;" << endl;
  report << "}" << endl;
  report << ".row {" << endl;
  report << "display: table-row;" << endl;
  report << "border:1px solid white;" << endl;
  report << "}" << endl;
  report << ".cell {" << endl;
  report << "display: table-cell;" << endl;
  report << "border:1px solid white;" << endl;
  report << "}" << endl;
  report << "</style>" << endl;
  report << "</head>" << endl;
  report << "<body>" << endl;
  report << "<script> function fresh() { location.reload(); } setInterval(\"fresh()\",60000); </script>" << endl;
  report << "<h1 align = center>Current status</h1>" << endl;

  report << "<p style=\"font-size: 200%; text-align: center;\">Poductivity: <font style=\"color:";
  float pfs = 0.0;
  if (hs->size() > 0) pfs = hs->at(hs->size()-1).pfs;
  int time_break = when_a_break ((vector <HUMAN_STATUS> *) hs, current_time);
  if (time_break > 88) pfs = 0.0;
  if (pfs > 0.50) report << "green";
  else if (pfs < 0.35) report << "red";
  else report << "yellow";
  report << ";\">" << endl;
  string pfs_string;
  if (pfs > 0.0) {
    pfs_string = pfs_string + to_string(pfs*100) + "%";
    pfs_string.replace(pfs_string.find("."),1,",");
  }
  else pfs_string = pfs_string + "undefined";
  report << pfs_string;
  report << ".</font><br>Before rest: <font style=\"color:";
  if (time_break >= 5) report << "green";
  else if (time_break < -5) report << "red";
  else report << "yellow";
  if (time_break < 0) time_break = 0;
  report << ";\">" << time_break << " min.</font><br>Worked out for the day: <font style=\"color:";
  int time_work = 0;
  int time_rest = 0;
  for (vector<HUMAN_STATUS>::iterator count = hs->begin();
       count!=hs->end(); count++) {
    if (count->intensity > 0) {
      if ((time_rest < 9) && (time_rest > 0)) {
        time_work += time_rest + 1;
        time_rest = 0;
      }
      else time_work++;
    }
    else {
      time_rest++;
    }
  }
  if (time_rest < 9) time_work += time_rest;

  if (time_work <= 450) report << "green";
  else if (time_work >= 480) report << "red";
  else report << "yellow";
  report << ";\">";
  report << time_work/60 << " h. " << time_work%60 << " min.</font></p>" << endl;
  report << "<p>&nbsp;</p>" << endl;

  report << "<h1 align = center>Daily report</h1>" << endl;
  report << "<div id=\"table\">" << endl;
  report << "<div class=\"row\">" << endl;
  report << "<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Start</span>" << endl;
  report << "<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Stop</span>" << endl;
  report << "<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Productivity</span>" << endl;
  report << "<span class=\"cell\" style=\"font-size: 120%; text-align: center;\">Type of work</span>" << endl;
  report << "</div>" << endl;

  vector <REPORT_RECORD> rr;
  rr.reserve(100);
  human_status_to_report_record (hs, (vector <REPORT_RECORD> *) &rr);
  for (vector<REPORT_RECORD>::iterator count = rr.begin();
       count!=rr.end(); count++) {
    string text_color;
    text_color.clear();
    if (count->rest_or_work == 0) text_color = "black";
    else if ((count->stop_time - count->stop_time)/60 < 100
             && (count->pfs > 0.5)) text_color = "green";
         else if ((count->stop_time - count->stop_time)/60 < 120
             && (count->pfs > 0.4)) text_color = "yellow";
              else text_color = "red";

    report << "<div class=\"row\">" << endl;
    report << "<span class=\"cell\" style=\"text-align:center;color:";
    report << text_color << ";\">";
    struct tm * timeinfo;
    timeinfo = localtime(&count->start_time);
    report << timeinfo->tm_mday << "." << timeinfo->tm_mon+1 << "." <<
              timeinfo->tm_year + 1900 << " " << timeinfo->tm_hour << ":" <<
              timeinfo->tm_min << ":" << timeinfo->tm_sec;
    report << "</span>" << endl;
    report << "<span class=\"cell\" style=\"text-align:center;color:" << endl;
    report << text_color << ";\">" << endl;
    timeinfo = localtime(&count->stop_time);
    report << timeinfo->tm_mday << "." << timeinfo->tm_mon+1 << "." <<
              timeinfo->tm_year + 1900 << " " << timeinfo->tm_hour << ":" <<
              timeinfo->tm_min << ":" << timeinfo->tm_sec;
    report << "</span>" << endl;
    report << "<span class=\"cell\" style=\"text-align:center;color:" << endl;
    report << text_color << ";\">" << endl;

    pfs_string.clear();
    if (count->pfs_count > 0) {
      pfs_string = pfs_string + to_string(count->pfs/count->pfs_count*100)+"%";
      pfs_string.replace(pfs_string.find("."),1,",");
    }
    else
      pfs_string = pfs_string + "undefined";
    if (count->rest_or_work == 1) report << pfs_string << "</span>" << endl;
    else report << "-</span>" << endl;

    report << "<span class=\"cell\" style=\"text-align:center;color:" << endl;
    report <<  text_color  << ";\">" << endl;
    if (count->rest_or_work == 1) {
      if (count->intensity > 10) report << MESSAGETYPEOUTPUT;
      else report << MESSAGETYPEINPUT;
    }
    else report << MESSAGETYPEREST;
    report << "</span>" << endl;
    report << "</div>" << endl;
  }

  report << "</div>" << endl;
  report << "</body>" << endl;
  report << "</html>" << endl;
  report.close();

}

void warning (char type) {
  string buff;
  buff.clear();
  buff = buff + "espeak  -vru -s130 \"";
  switch (type) {
    case 1: {buff = buff + "5 minutes before the rest\""; break;}
    case 2: {buff = buff + "Warning. Rest. Repeat. Warning. Rest.\"";
              break;}
    case 3: {buff = buff + "The break is over.\""; break;}
    default: buff = buff + "Work\"";
  }
  int result = system(buff.c_str());
  result++;
}

int when_a_break (vector <HUMAN_STATUS> * hs, time_t current_time) {
  int time_break = 0;
  int time_rest = 0;
  time_t old_temp_time = 0;
  time_t temp_time = 0;
  for (vector<HUMAN_STATUS>::reverse_iterator count = hs->rbegin();
    count!=hs->rend(); count++) {
    temp_time = count->current_time;
    int space_time = 0;
    if (old_temp_time == 0) space_time = 1;
    else space_time = (old_temp_time - temp_time)/60;
    old_temp_time = temp_time;
    if ((count->intensity == 0) || (space_time > 1)) {
      time_rest += space_time;
    }
    else {
      if (time_rest != 0) {
        time_break += time_rest;
        time_rest = 0;
      }
      time_break++;
    }
    if (time_rest > 9) break;
    if (current_time - 18000 > count->current_time) break;
  }
  time_break = 90 - time_break;
  return time_break;
}

void human_status_to_report_record (vector <HUMAN_STATUS> * hs,
                                    vector <REPORT_RECORD> * rr) {
  struct REPORT_RECORD srr = {0};
  time_t current_time_old = 0;
  for ( vector<HUMAN_STATUS>::reverse_iterator count = hs->rbegin();
        count!=hs->rend(); count++) {
    if ((current_time_old - count->current_time > 62) &&
        (current_time_old)) {
      srr.start_time = count->current_time;
      srr.stop_time = current_time_old - 60;
      srr.intensity = 0;
      srr.rest_or_work = 0;
      srr.pfs = 0;
      srr.pfs_count = 0;
      rr->push_back(srr);
    }
    srr.start_time = count->current_time - 60;
    srr.stop_time = count->current_time;
    srr.intensity = count->intensity;
    if (count->intensity > 0) {
      srr.rest_or_work = 1;
      srr.pfs = count->pfs;
      srr.pfs_count = 1;
    }
    else {
      srr.rest_or_work = 0;
      srr.pfs = 0;
      srr.pfs_count = 0;
    }
    current_time_old = count->current_time;
    rr->push_back(srr);
  }
  vector<REPORT_RECORD>::iterator count = rr->begin();
  count = rr->begin();
  do {
    if (count == rr->begin()) {
      if (rr->size() > 1) count++;
      else break;
    }
    if ((count->intensity == 0) && ((count-1)->intensity == 0)) {
      count->start_time = count->start_time;
      count->stop_time = (count-1)->stop_time;
      count->intensity = 0;
      count->rest_or_work = 0;
      count->pfs = 0;
      count->pfs_count = 0;
      rr->erase(count-1);
    }
    else count++;
  }
  while (count!=rr->end());
  count = rr->begin();
  do {
    if (count == rr->begin()) {
      if (rr->size() > 1) count++;
      else break;
    }
    if ((count->intensity > 0) && ((count-1)->intensity > 0)) {
      count->start_time = count->start_time;
      count->stop_time = (count-1)->stop_time;
      count->intensity += (count-1)->intensity;
      count->rest_or_work = 1;
      count->pfs += (count-1)->pfs;
      count->pfs_count += (count-1)->pfs_count;
      rr->erase(count-1);
    }
    else count++;
  }
  while (count!=rr->end());
  count = rr->begin();
  do {
    if (count == rr->begin()) {
      if (rr->size() > 1) count++;
      else break;
    }
    if ((count->intensity > 0) && ((count-1)->intensity == 0) &&
        (((count-1)->stop_time - (count-1)->start_time))/60 < 9) {
      count->start_time = count->start_time;
      count->stop_time = (count-1)->stop_time;
      count->intensity += (count-1)->intensity;
      count->rest_or_work = 1;
      count->pfs += (count-1)->pfs;
      count->pfs_count += (count-1)->pfs_count;
      rr->erase(count-1);
    }
    else count++;
  }
  while (count!=rr->end());
  count = rr->begin();
  do {
    if (count == rr->begin()) {
      if (rr->size() > 1) count++;
      else break;
    }
    if ((count->intensity > 0) && ((count-1)->intensity > 0)) {
      count->start_time = count->start_time;
      count->stop_time = (count-1)->stop_time;
      count->intensity += (count-1)->intensity;
      count->rest_or_work = 1;
      count->pfs += (count-1)->pfs;
      count->pfs_count += (count-1)->pfs_count;
      rr->erase(count-1);
    }
    else count++;
  }
  while (count!=rr->end());
}
