/*Copyright (C) 2017 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#define SOURCE_DATA_SIZE 5000
#define DOUBLE_INTENSITY_SIZE 500
#define DOUBLE_PRESS_SIZE 1000
#define DOUBLE_ETALON_SIZE 500
#define CALCULATE_TIME 60
#define OS_TYPE 1
#define SIGNIFICANT_EXCESS 3000
#define ETALON_FLUCTUATION

struct KEYPRESS {
  time_t current_time;
  long count;
  int key;
  char up_down;
};

struct DOUBLE_INTENSITY {
  int key;
  int key_second;
  long intensity;
};

struct DOUBLE_PRESS {
  time_t current_time;
  long count;
  int key;
  int key_up;
  int key_second;
  int key_second_down;
  int key_second_up;
};

struct DOUBLE_ETALON {
  int intensity;
  int key1;
  int retaining1;
  int retaining1_buffer[DOUBLE_PRESS_SIZE];
  int space1;
  int space1_buffer[DOUBLE_PRESS_SIZE];
  int key2;
  int retaining2;
  int retaining2_buffer[DOUBLE_PRESS_SIZE];
};

struct KEYPRESS * source_data;
struct DOUBLE_INTENSITY * di;
struct DOUBLE_PRESS * dp;
struct DOUBLE_ETALON * de;
int keypress_count;

char * file_events;
char * file_etalon;

int add_source_record (time_t current_time, long count,
                       int key, char up_down, struct KEYPRESS * source_data);
int transformation (struct KEYPRESS * source_data, struct DOUBLE_INTENSITY * di,
                    struct DOUBLE_PRESS * dp, struct DOUBLE_ETALON * de);
int source_data_to_double_press(struct KEYPRESS * source_data, struct DOUBLE_INTENSITY * di);
int dump (char * filename, struct DOUBLE_PRESS * dp, struct DOUBLE_INTENSITY * di,
                                                        struct DOUBLE_ETALON * de);
int current_time_count_different (time_t t1, long c1, time_t t2, long c2, char OS);
int double_press_to_intensity(struct DOUBLE_PRESS * dp, struct DOUBLE_INTENSITY * di);
int create_image(struct DOUBLE_PRESS * dp, struct DOUBLE_INTENSITY * di,
                 struct DOUBLE_ETALON * de);
int save_etalon (char * filename, struct DOUBLE_ETALON * de);

int main (int argc, char * argv[]) {
  source_data = (struct KEYPRESS*) calloc (SOURCE_DATA_SIZE,sizeof(struct KEYPRESS));
  di = (struct DOUBLE_INTENSITY*) calloc (DOUBLE_INTENSITY_SIZE,sizeof(struct DOUBLE_INTENSITY));
  dp = (struct DOUBLE_PRESS*) calloc (DOUBLE_PRESS_SIZE,sizeof(struct DOUBLE_PRESS));
  de = (struct DOUBLE_ETALON*) calloc (DOUBLE_ETALON_SIZE,sizeof(struct DOUBLE_ETALON));
  int function_result = 0;

  file_events = argv[1];
  file_etalon = argv[2];

  keypress_count = 0;

  FILE * source_file;
  source_file = fopen(file_events,"r");
  if (source_file == NULL) {
    free (source_data);
    free(di);
    free(dp);
    free(de);
    return 1;
  }
  struct KEYPRESS kp = {0};
  char read_buffer [30] = {0};
  while(1) {
    char * rb = fgets ((char *)&read_buffer, 29, source_file);
    if (feof (source_file)) break;
    rb = (char *)&read_buffer;
    char * pch = strtok (rb," ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.current_time = atol (pch);
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.count = atol (pch);
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.key = atoi(pch);
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.up_down = atoi (pch);
    if (kp.up_down == 0) keypress_count++;
    if ((kp.current_time != 0) && (kp.count != 0)) {
      function_result = add_source_record (kp.current_time, kp.count, kp.key, 
                                           kp.up_down, source_data);
    }
    bzero(read_buffer,sizeof(char)*30);
  }
  fclose(source_file);

  function_result = transformation (source_data, di, dp, de);

  function_result = save_etalon (file_etalon, de);

  if (function_result != 0) puts("Error in function dump()!");

  free (source_data);
  free(di);
  free(dp);
  free(de);
  return 0;
};

int save_etalon (char * filename, struct DOUBLE_ETALON * de) {
  FILE * dump_file = fopen(filename,"w");
  if (dump_file == NULL) return 1;
  int i = 0;
  fprintf(dump_file, "%i\n", keypress_count);
  for (i = 0; i < DOUBLE_ETALON_SIZE; i++) {  
    if (de[i].intensity == 0) break;
    fprintf(dump_file, "%i %i %i %i %i %i\n", de[i].intensity, de[i].key1,
                        de[i].retaining1, de[i].space1, de[i].key2, de[i].retaining2);
  }
  fclose(dump_file);
  return 0;
};

int current_time_count_different (time_t t1, long c1, time_t t2, long c2, char OS) {
  long t1_ = 0l;
  long c1_ = c1;
  long t2_ = 0l;
  long c2_ = c2;
  if (OS == 1) {
    c1_ /= 1000;
    c2_ /= 1000;
  }
  t1_ = t1 * 1000 + c1_;
  t2_ = t2 * 1000 + c2_;
  int result = t1_ - t2_;
  return result;
}

int add_source_record (time_t current_time, long count,
                       int key, char up_down, struct KEYPRESS * source_data) {
  int i = 0;
  for (i = 0; i < SOURCE_DATA_SIZE; i++) {
    if (source_data[i].current_time == 0) {
      source_data[i].current_time = current_time;
      source_data[i].count = count;
      source_data[i].key = key;
      source_data[i].up_down = up_down;
      break;
    }
  }
  return 0;
}

int transformation (struct KEYPRESS * source_data, struct DOUBLE_INTENSITY * di,
                    struct DOUBLE_PRESS * dp, struct DOUBLE_ETALON * de) {
  source_data_to_double_press(source_data, di);
  double_press_to_intensity(dp, di);
  create_image(dp, di, de);
  return 0;
}

int dump (char * filename, struct DOUBLE_PRESS * dp,
          struct DOUBLE_INTENSITY * di, struct DOUBLE_ETALON * de) {
  FILE * dump_file = fopen(filename,"w");
  int i = 0;
  for (i = 0; i < DOUBLE_PRESS_SIZE; i++) {
    if ((dp[i].current_time == 0) && (dp[i].count == 0) && (dp[i].key == 0) &&
        (dp[i].key_up == 0) && (dp[i].key_second == 0) &&
        (dp[i].key_second_down == 0) && (dp[i].key_second_up == 0)) break;
      fprintf(dump_file, "%li %li %i %i %i %i %i\n", dp[i].current_time, dp[i].count,
           dp[i].key, dp[i].key_up, dp[i].key_second,
           dp[i].key_second_down, dp[i].key_second_up);
  }
  for (i = 0; i < DOUBLE_INTENSITY_SIZE; i++) {  
    if (di[i].intensity == 0) break;
    fprintf(dump_file, "%i %i %li\n", di[i].key, di[i].key_second, di[i].intensity);
  }
  for (i = 0; i < DOUBLE_ETALON_SIZE; i++) {  
    if (de[i].intensity == 0) break;
    fprintf(dump_file, "%i %i %i %i %i %i\n", de[i].intensity, de[i].key1,
                        de[i].retaining1, de[i].space1, de[i].key2, de[i].retaining2);
  }
  fclose(dump_file);
  return 0;
}

int source_data_to_double_press(struct KEYPRESS * source_data, struct DOUBLE_INTENSITY * di) {
  time_t current_time  = time(NULL);
  bzero(dp,sizeof(struct DOUBLE_PRESS)*DOUBLE_PRESS_SIZE);  
  int i = 0;
  for (i = 0; i < SOURCE_DATA_SIZE; i++) {
    if ((current_time - source_data[i].current_time) > CALCULATE_TIME) {
      bzero ((void *)&source_data[i],sizeof(struct KEYPRESS));
      continue;
    }
    if (source_data[i].up_down == 2) {
      bzero ((void *)&source_data[i],sizeof(struct KEYPRESS));
      continue;
    }
    int j = 0;
    if (source_data[i].up_down == 1) {
      for (j = 0; j < DOUBLE_PRESS_SIZE; j++) {
        if (dp[j].current_time == 0) {
          dp[j].current_time = source_data[i].current_time;
          dp[j].count = source_data[i].count;
          dp[j].key = source_data[i].key;
          break;
        }
      }
      for (j = 0; j < DOUBLE_PRESS_SIZE; j++) {      
        if ((dp[j].current_time != 0) && (dp[j].key_second_down == 0)) {
          if ((dp[j].current_time != source_data[i].current_time) ||
                              (dp[j].count != source_data[i].count)) {
            dp[j].key_second = source_data[i].key;
            dp[j].key_second_down = current_time_count_different
                       (source_data[i].current_time, source_data[i].count,
                        dp[j].current_time, dp[j].count, OS_TYPE);
          }
        }
      }
    }
    else {
      for (j = 0; j < DOUBLE_PRESS_SIZE; j++) {
        if ((dp[j].current_time != 0) && (dp[j].key_up == 0) &&
           (source_data[i].up_down == 0) && (dp[j].key == source_data[i].key)) {
          dp[j].key_up = current_time_count_different (source_data[i].current_time,
                    source_data[i].count, dp[j].current_time, dp[j].count, OS_TYPE);
        }
      }
      for (j = 0; j < DOUBLE_PRESS_SIZE; j++) {
        if ((dp[j].current_time != 0) && (dp[j].key_second_up == 0) &&
        (source_data[i].up_down == 0) && (dp[j].key_second == source_data[i].key)) {
          dp[j].key_second_up = current_time_count_different (source_data[i].current_time,
                             source_data[i].count, dp[j].current_time, dp[j].count, OS_TYPE);
        }
      }
    }
  }
  for (i = 0; i < DOUBLE_PRESS_SIZE; i++) {
    if (dp[i].current_time == 0) continue;
    if (dp[i].key_second_up > SIGNIFICANT_EXCESS)
      bzero ((void *)&dp[i],sizeof(struct DOUBLE_PRESS));
  }
  int max_value = DOUBLE_PRESS_SIZE-1;
  for (i = 0; i < max_value; i++) {
    if (dp[i].current_time == 0) {
      int count = 0;
      for (count = i; count < max_value; count++) {
        memcpy(&dp[count],&dp[count+1],sizeof(struct DOUBLE_PRESS));
      }
    }
  }
  return 0;
};

int double_press_to_intensity(struct DOUBLE_PRESS * dp, struct DOUBLE_INTENSITY * di) {
  bzero(di,sizeof(struct DOUBLE_INTENSITY)*DOUBLE_INTENSITY_SIZE);
  int count = 0;
  int i = 0;
  for (count = 0; count < DOUBLE_PRESS_SIZE; count++) {
    for (i = 0; i < DOUBLE_INTENSITY_SIZE; i++) {
      if (dp[count].current_time == 0) break;
      if ((dp[count].key == di[i].key) && (dp[count].key_second == di[i].key_second)) {
        di[i].intensity++;
        break;
      }
      if (di[i].intensity == 0) {
        di[i].key = dp[count].key;
        di[i].key_second = dp[count].key_second;
        di[i].intensity = 1;
        break;
      }
    }
  }

  struct DOUBLE_INTENSITY di_temp = {0};
  for (i = 0; i < DOUBLE_INTENSITY_SIZE; i++) {
    for (count = 0; count < DOUBLE_INTENSITY_SIZE; count++) {
      if (di[i].intensity == 0) continue;
      if (di[i].intensity > di[count].intensity) {
        memcpy(&di_temp,&di[count],sizeof(struct DOUBLE_INTENSITY));
        memcpy(&di[count],&di[i],sizeof(struct DOUBLE_INTENSITY));
        memcpy(&di[i],&di_temp,sizeof(struct DOUBLE_INTENSITY));
      }
    }
  }
  for (i = 0; i < DOUBLE_INTENSITY_SIZE; i++) {
    if (di[i].intensity < 2)
      bzero ((void *)&di[i],sizeof(struct DOUBLE_INTENSITY));
  }
  return 0;
};

int create_image(struct DOUBLE_PRESS * dp, struct DOUBLE_INTENSITY * di,
                 struct DOUBLE_ETALON * de) {
  int i, j;
  for (i = 0; i < DOUBLE_INTENSITY_SIZE; i++) {
    if (di[i].intensity == 0) continue;
    for (j = 0; j < DOUBLE_ETALON_SIZE; j++) {
      if (de[j].intensity == 0) {
        de[j].key1 = di[i].key;
        de[j].key2 = di[i].key_second;
        de[j].intensity = di[i].intensity;
        break;
      }
    }
  }
  for (i = 0; i < DOUBLE_PRESS_SIZE; i++) {
    for (j = 0; j < DOUBLE_ETALON_SIZE; j++) {
      if ((dp[i].key == de[j].key1) && (dp[i].key_second == de[j].key2)) {
        int count;
        for (count = 0; count < DOUBLE_PRESS_SIZE; count++) {
          if ((de[j].retaining1_buffer[count] == 0) && (de[j].space1_buffer[count] == 0) &&
              (de[j].retaining2_buffer[count] == 0)) {
            de[j].retaining1_buffer[count] = dp[i].key_up;
            de[j].space1_buffer[count] = dp[i].key_second_down - dp[i].key_up;
            de[j].retaining2_buffer[count] = dp[i].key_second_up - dp[i].key_second_down;
            break;
          }
        }
      }
    }
  }

  for (i = 0; i < DOUBLE_ETALON_SIZE; i++) {
    int count;
    int summ_count = 0;
    int summ_retaining1, summ_space1, summ_retaining2;
    summ_retaining1 = summ_space1 = summ_retaining2 = 0;
    for (count = 0; count < DOUBLE_PRESS_SIZE; count++) {
      if (de[i].retaining1_buffer[count] == 0) break;
      summ_retaining1 += de[i].retaining1_buffer[count];
      summ_space1 += de[i].space1_buffer[count];
      summ_retaining2 += de[i].retaining2_buffer[count];
      summ_count++;
      break;
    }
    if (summ_count == 0) summ_count = 1;
    de[i].retaining1 = summ_retaining1 / summ_count;
    de[i].space1 = summ_space1 / summ_count;
    de[i].retaining2 = summ_retaining2 / summ_count;
  }
  return 0;
};
