/*Copyright (C) 2017 Abashin Valerii Gennadievich <valeriy@abashin.ru>

This file is part of abashin.ru software.

Abashin.ru is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

Abashin.ru is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Abashin.ru. If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <string.h>

#define MOUSE_PRESS_SIZE 1000
#define SOURCE_DATA_SIZE 5000
#define MOUSEKEYS 3
#define LKM 272
#define RKM 273
#define MKM 274

int mousepress_count;

char * file_events;
char * file_etalon;

struct KEYPRESS {
  time_t current_time;
  long count;
  int key;
  char up_down;
};

struct MOUSEPRESSETALON {
  int key;
  int intensity;
  int retaining1;
  int retaining1_buffer[MOUSE_PRESS_SIZE];
};

struct KEYPRESS * source_data;
struct MOUSEPRESSETALON * mouse_etalon;

int save_etalon (char * filename, struct MOUSEPRESSETALON * mouse_etalon, int mousepress_count);
int current_time_count_different (time_t t1, long c1, time_t t2, long c2, char OS);
int add_source_record (time_t current_time, long count,
                       int key, char up_down, struct KEYPRESS * source_data);
int transformation(struct KEYPRESS * source_data, struct MOUSEPRESSETALON * mouse_etalon);

int main (int argc, char * argv[]) {
  source_data = (struct KEYPRESS*) calloc (SOURCE_DATA_SIZE,sizeof(struct KEYPRESS));
  mouse_etalon = (struct MOUSEPRESSETALON*) calloc (MOUSEKEYS,sizeof(struct MOUSEPRESSETALON));

  int function_result = 0;

  file_events = argv[1];
  file_etalon = argv[2];

  mousepress_count = 0;

  FILE * source_file;
  source_file = fopen(file_events,"r");
  if (source_file == NULL) {
    return 1;
  }
  struct KEYPRESS kp = {0};
  char read_buffer [30] = {0};
  while(1) {
    char * rb = fgets ((char *)&read_buffer, 29, source_file);
    if (feof (source_file)) break;
    rb = (char *)&read_buffer;
    char * pch = strtok (rb," ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.current_time = atol (pch);
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.count = atol (pch);
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.key = atoi(pch);
    pch = strtok (NULL, " ");
    if (pch == NULL) {
      bzero(read_buffer,sizeof(char)*30);
      continue;
    }
    kp.up_down = atoi (pch);
    if ((kp.up_down == 0) && (kp.key == 272 || kp.key == 273 || kp.key == 274))
          mousepress_count++;
    if ((kp.current_time != 0) && (kp.count != 0)) {
      function_result = add_source_record (kp.current_time, kp.count, kp.key, 
                                           kp.up_down, source_data);
    }
    bzero(read_buffer,sizeof(char)*30);
  }
  fclose(source_file);
  transformation(source_data, mouse_etalon);
  function_result = save_etalon (file_etalon, mouse_etalon, mousepress_count);
  if (function_result != 0) puts("Error in function dump()!");

  free (source_data);
  free (mouse_etalon);
  return 0;
};

int save_etalon (char * filename, struct MOUSEPRESSETALON * mouse_etalon, int mousepress_count) {
  FILE * dump_file = fopen(filename,"w");
  if (dump_file == NULL) return 1;
  fprintf(dump_file, "%i\n", mousepress_count);
  int i = 0;
  for (i = 0; i < MOUSEKEYS; i++) {  
    if (mouse_etalon[i].intensity == 0) break;
    fprintf(dump_file, "%i %i %i\n", mouse_etalon[i].intensity, mouse_etalon[i].key,
                        mouse_etalon[i].retaining1);
  }
  fclose(dump_file);
  return 0;
};

int current_time_count_different (time_t t1, long c1, time_t t2, long c2, char OS) {
  long t1_ = 0l;
  long c1_ = c1;
  long t2_ = 0l;
  long c2_ = c2;
  if (OS == 1) {
    c1_ /= 1000;
    c2_ /= 1000;
  }
  t1_ = t1 * 1000 + c1_;
  t2_ = t2 * 1000 + c2_;
  int result = t1_ - t2_;
  return result;
}

int add_source_record (time_t current_time, long count,
                       int key, char up_down, struct KEYPRESS * source_data) {
  int i = 0;
  for (i = 0; i < SOURCE_DATA_SIZE; i++) {
    if (source_data[i].current_time == 0) {
      source_data[i].current_time = current_time;
      source_data[i].count = count;
      source_data[i].key = key;
      source_data[i].up_down = up_down;
      break;
    }
  }
  return 0;
}

int transformation (struct KEYPRESS * source_data, struct MOUSEPRESSETALON * mouse_etalon) {
  int count = 0;
  for (count = 0; count < SOURCE_DATA_SIZE; count++) {
    if ((source_data[count].current_time != 0) && (source_data[count].up_down != 2)) {
      if (source_data[count].up_down == 1) {
        int count2 = count + 1;
        int count2_max = SOURCE_DATA_SIZE - 1;
        for (count2 = count; count2 < count2_max; count2++) {
          if ((source_data[count2].up_down == 0) && (source_data[count2].key ==
               source_data[count].key)) {
            source_data[count].count = current_time_count_different (source_data[count2].current_time,
                                       source_data[count2].count, source_data[count].current_time,
                                       source_data[count].count, 1);
            break;
          }
        }
        source_data[count2].current_time = 0;
        source_data[count2].count = 0;
        source_data[count2].key = 0;
        source_data[count2].up_down = 0;
        source_data[count].up_down = 2;
      }
    }
  }
  mouse_etalon[0].key = LKM;
  mouse_etalon[1].key = RKM;
  mouse_etalon[2].key = MKM;

  for (count = 0; count < SOURCE_DATA_SIZE; count++) {
    if (source_data[count].up_down == 2) {
      int count_key = 0;
      for (count_key = 0; count_key < MOUSEKEYS; count_key++) {
        if (source_data[count].key == mouse_etalon[count_key].key) {
          int count_press = 0;
          for (count_press = 0; count_press < MOUSE_PRESS_SIZE; count_press++) {
            if (mouse_etalon[count_key].retaining1_buffer[count_press] == 0) {
              mouse_etalon[count_key].retaining1_buffer[count_press] =
                                                    source_data[count].count;
              break;
            }
          }
        }
      }
    }
  }
  int i = 0;
  for (i = 0; i < MOUSEKEYS; i++) {
    int count;
    int summ_count = 0;
    int summ_retaining1 = 0;
    for (count = 0; count < MOUSE_PRESS_SIZE; count++) {
      if (mouse_etalon[i].retaining1_buffer[count] == 0) break;
      summ_retaining1 += mouse_etalon[i].retaining1_buffer[count];
      summ_count++;
    }
    mouse_etalon[i].intensity = summ_count;
    if (summ_count == 0) summ_count = 1;
    mouse_etalon[i].retaining1 = summ_retaining1 / summ_count;
  }
  return 0;
}
